 //Uber Board v2
//Sample code, 6/6/08



/* *********************************************************
               Function declarations
  ********************************************************* */



void Initialize(void);
void feed(void);



void stat0_on (void);
void stat0_off (void);

void stat1_on (void);
void stat1_off (void);

void stat2_on (void);
void stat2_off (void);

void put_char(char c);
char getc(void);

void setup_uart0(int baud);
void setup_uart1(int baud);

void set_mux(void);
void set_sense(void);

void config_menu(void);

void get_adc(void);


/**********************************************************
                  Header files
 **********************************************************/
#include "LPC214x.h"
#include "rprintf.h"
#include "fat16.h"
#include "sd_raw.h"


/**********************************************************
                  Global Variables
 **********************************************************/
char port = 0;        //determines which port will be written to
static int x_sense = 0, y_sense = 0, z_sense = 0;
char channel = 0;
char sense = 0;
char temp_baud = 6;
char EM_408_power = 0;


/**********************************************************
                       MAIN
**********************************************************/

int	main (void)
{	
	int	j = 0,m = 0;
    
	Initialize();	

	for (j = 0; j < 5; j++)
	{
		stat0_on();
		for (m = 0; m < 200000; m++);
		stat0_off();
		stat1_on();
		for (m = 0; m < 200000; m++);
		stat1_off();
		stat2_on();
		for (m = 0; m < 200000; m++);
		stat2_off();
		
	}
	
	rprintf_devopen(put_char); /* init rrprintf */
	
	setup_uart0(115200);
	rprintf("UART0 setup...\r\n",0);

    setup_uart1(115200);
	
	config_menu();
	
	while(1);
        
        

		
}



/**********************************************************
                      Initialize
**********************************************************/

#define PLOCK 0x400

void Initialize(void)
{
        // Setting Multiplier and Divider values
  	PLLCFG=0x23;
  	feed();
  
	// Enabling the PLL */	
	PLLCON=0x1;
	feed();
  
	// Wait for the PLL to lock to set frequency
	while(!(PLLSTAT & PLOCK)) ;
  
	// Connect the PLL as the clock source
	PLLCON=0x3;
	feed();
  
	// Enabling MAM and setting number of clocks used for Flash memory fetch (4 cclks in this case)
	MAMCR=0x2;
	MAMTIM=0x4;
  
	// Setting peripheral Clock (pclk) to System Clock (cclk)
	VPBDIV=0x01;
	
	/*
	P0.28/AD0.1 = X_Sense
	P0.29/AD0.2 = Y_Sense
	P0.30/AD0.3 = Z_Sense
	*/
	
	PINSEL0 = 0x00050005;   // enable uart0, enable uart1
	PINSEL0 |= 0x00001500;   // enable MISO, MOSI, SCK for SPI0 (CS will be an output)
	
	PINSEL1 = 0x15000000;	//AD0.1, AD0.2, AD0.3 enabled
							
	IODIR0 = 0x001C0000;    //P0.18 = STAT0, P0.19 = STAT1, P0.20 = STAT2
	IODIR0 |= 0x00010080;	//P0.16 is "IN2" on Mux, P0.7 is CS for SPI0
	IODIR0 |= 0x00008000;	//P0.15 is mux_inhibit
	IODIR0 |= 0x00800000;	//P0.23 is EM-408 power enable
	IODIR0 |= 0x00002000;	//P0.13 is cell_on, high active
	
	IODIR1 = 0x00030000;	//GS1 and GS2 set as outputs
	IODIR1 |= 0x00400000;	//P1.22 is "IN2" on Mux
	IODIR1 |= 0x00200000;	//P1.21 is BT_CTS

	IOSET0 = 0x001C0000;	//Stat LEDs are low active, set high = off
	
	IOCLR0 = 0x00030000;	//GS1 and GS2 set low
	IOCLR0 = 0x00010000;	//Mux IN2 set low
	IOCLR0 = 0x00008000;	//mux_inhibit set low for operation
	IOCLR0 = 0x00800000;	//EM-408 is off
	
	IOCLR1 = 0x00400000;	//Mux IN1 set low
	IOCLR1 = 0x00200000;	//BT_CTS set low
	IOCLR1 = 0x00100000;	//cell_on set low
       
	
}


void feed(void)
{
  PLLFEED=0xAA;
  PLLFEED=0x55;
}


void stat0_on (void)
{
	IOCLR0 = 0x00040000;

}

void stat0_off (void)
{
	IOSET0 = 0x00040000;
	
}

void stat1_on (void)
{
	IOCLR0 = 0x00080000;

}

void stat1_off (void)
{
	IOSET0 = 0x00080000;
	
}

void stat2_on (void)
{
	IOCLR0 = 0x00100000;

}

void stat2_off (void)
{
	IOSET0 = 0x00100000;
	
}

void put_char(char c)
{
    if (port == 0)
    {
        while((U0LSR & 0x20) == 0);
        U0THR = c;
    }
    else if (port == 1)
    {
        while((U1LSR & 0x20) == 0);
        U1THR = c;
    }
}

//specific to UART0
char getc(void)
{
    while((U0LSR & 0x01) == 0);
    return (U0RBR);
}


void setup_uart0(int baud)
{
	
	//set up uart0 
	U0LCR = 0x83;			// 8 bits, no Parity, 1 Stop bit, DLAB = 1
	
	
	if (baud == 1200)
	{
		U0DLM   = 0x09;		  	//
  		U0DLL 	= 0xC0;        	// 1200 Baud Rate @ 48MHz VPB Clock
	}
	
	if (baud == 2400)
	{
		U0DLM   = 0x04;		  	//
  		U0DLL 	= 0xE0;        	// 2400 Baud Rate @ 48MHz VPB Clock
	}
		
	if (baud == 4800)
	{
		U0DLM   = 0x02;		  	//
  		U0DLL 	= 0x70;        	// 4800 Baud Rate @ 48MHz VPB Clock
	}
	
	else if (baud == 9600)
	{
	 	U0DLM   = 0x01;		  	//
  		U0DLL 	= 0x38;        	// 9600 Baud Rate @ 48MHz VPB Clock
	}
	
	else if (baud == 19200)
	{
	 	U0DLM   = 0x00;		  	//
  		U0DLL 	= 0x9C;        	// 19200 Baud Rate @ 48MHz VPB Clock
	}
	
	else if (baud == 38400)
	{
	 	U0DLM   = 0x00;		  	//
  		//U0DLL 	= 0x60;        	// 38400 Baud Rate @ 48MHz VPB Clock
		U0DLL 	= 0x4E;
	}

        else if (baud == 57600)
	{
	 	U0DLM   = 0x00;		  	//
  		//U0DLL 	= 0x40;        	// 57600 Baud Rate @ 48MHz VPB Clock
		U0DLL 	= 0x34;
	}

        else if (baud == 115200)
	{
	 	U0DLM   = 0x00;		  	//
  		//U0DLL 	= 0x20;        	// 115200 Baud Rate @ 48MHz VPB Clock
		U0DLL 	= 0x1A;
	}
	                    
  	U0FCR	= 0x01;			//
	U0LCR 	= 0x03;        	// DLAB = 0
	
	
	
}


void setup_uart1(int baud)
{
	//set up uart1
	U1LCR 	= 0x83;			// 8 bits, no Parity, 1 Stop bit, DLAB = 1
	
	if (baud == 1200)
	{
		U1DLM   = 0x09;		  	//
  		U1DLL 	= 0xC0;        	// 1200 Baud Rate @ 48MHz VPB Clock
	}
	
	if (baud == 2400)
	{
		U1DLM   = 0x04;		  	//
  		U1DLL 	= 0xE0;        	// 2400 Baud Rate @ 48MHz VPB Clock
	}
	
	if (baud == 4800)
	{
		U1DLM   = 0x02;		  	//
  		U1DLL 	= 0x70;        	// 4800 Baud Rate @ 48MHz VPB Clock
	}
	
	else if (baud == 9600)
	{
	 	U1DLM   = 0x01;		  	//
  		U1DLL 	= 0x38;        	// 9600 Baud Rate @ 48MHz VPB Clock
	}
	
	else if (baud == 19200)
	{
	 	U1DLM   = 0x00;		  	//
  		U1DLL 	= 0x9C;        	// 19200 Baud Rate @ 48MHz VPB Clock
	}
	
	else if (baud == 38400)
	{
	 	U1DLM   = 0x00;		  	//
  		U1DLL 	= 0x4E;        	// 38400 Baud Rate @ 48MHz VPB Clock
	}

        else if (baud == 57600)
	{
	 	U1DLM   = 0x00;		  	//
  		U1DLL 	= 0x34;        	// 57600 Baud Rate @ 48MHz VPB Clock
	}

        else if (baud == 115200)
	{
	 	U1DLM   = 0x00;		  	//
  		U1DLL 	= 0x1A;        	// 115200 Baud Rate @ 48MHz VPB Clock
	}
	       
  	U1FCR	= 0x01;			// 
  	
	U1LCR 	= 0x03;        	// DLAB = 0
	
	

}

void config_menu(void)
{
	char a, q;
	int m;
	
	stat0_on();
	stat1_on();
	stat2_on();

    //port = 1;

	while (1)
	{
		rprintf("\r\nUber Board V2 Test Code\r\n",0);
		rprintf("========================================\r\n",0);
		rprintf("1) Accelerometer/ADC\r\n",0);
		rprintf("2) Toggle accelerometer sensitivity, currently ",0);
		if (sense == 0) rprintf("1.5g\r\n",0);
		else if (sense == 1) rprintf("2g\r\n",0);
		else if (sense == 2) rprintf("4g\r\n",0);
		else rprintf("6g\r\n",0);
		rprintf("3) Toggle Mux Channel, currently ",0);
		if (channel == 0) rprintf("GPS\r\n",0);
		else if (channel == 1) rprintf("cell\r\n",0);
		else if (channel == 2) rprintf("Bluetooth port\r\n",0);
		else if (channel == 3) rprintf("Aux port\r\n",0);
		else rprintf("not connected\r\n",0);
		rprintf("4) Initialize SD card\r\n",0);
		rprintf("5) Toggle UART1 baud rate, currently ",0);
		if (temp_baud == 0) rprintf("4800\r\n",0);
		else if (temp_baud == 1) rprintf("9600\r\n",0);
		else if (temp_baud == 2) rprintf("19200\r\n",0);
		else if (temp_baud == 3) rprintf("38400\r\n",0);
		else if (temp_baud == 4) rprintf("57600\r\n",0);
		else rprintf("115200\r\n",0);
		rprintf("6) Toggle EM-408 power, currently ",0);
		if (EM_408_power == 0) rprintf("off\r\n",0);
		else if (EM_408_power == 1) rprintf("on\r\n",0);
		rprintf("7) Send power pulse to the cell module\r\n",0);
		rprintf("8) Talk to UART1 (press ^s to exit)\r\n",0);
	
		
		a = getc();
		
		if (a == '1')
        {
			while(1)
			{
				get_adc();
				
				rprintf("\r\nx = %d",x_sense);
				put_char(9);
				rprintf("y = %d",y_sense);
				put_char(9);
				rprintf("z = %d",z_sense);
				
				if((U0LSR & 0x01) == 1) a = U0RBR;
				if (a == 32) break;
				
			}
		}
		
		else if (a == '2')
		{
			sense++;
			if (sense >= 4) sense = 0;
			set_sense();
		}
		
		else if (a == '3')
		{
			channel++;
			if (channel >= 4) channel = 0;
			set_mux();
		}
		
		else if (a == '4')
		{
			rprintf("\r\n\r\n",0);
			m = sd_raw_init();
			if (m == 1) rprintf("Success\r\n",0);
			else if (m == 0) rprintf("Failed\r\n",0);
		}
		
		else if (a == '5')
		{
			temp_baud++;
			if (temp_baud >= 6) temp_baud = 0;
			
			if (temp_baud == 0) setup_uart1(4800);
			else if (temp_baud == 1) setup_uart1(9600);
			else if (temp_baud == 2) setup_uart1(19200);
			else if (temp_baud == 3) setup_uart1(38400);
			else if (temp_baud == 4) setup_uart1(57600);
			else setup_uart1(115200);
		}
		
		else if (a == '6')
		{
			EM_408_power ^= 1;;
			if (EM_408_power == 0) IOCLR0 = 0x00800000;
			else if (EM_408_power == 1) IOSET0 = 0x00800000;
		}
		
		else if (a == '7')
		{
			IOSET0 = 0x00002000;
			for (m = 0; m < 4500000; m++);
			IOCLR0 = 0x00002000;
			rprintf("\r\n...Sent...\r\n",0);
		}
		
		else if (a == '8')
		{
			while(1)
			{
				if ((U0LSR & 0x01) != 0)	//if something comes in on port0...
				{				
					q = U0RBR;	//gotta read the RX buffer to set the flag back to 0
				  
					if (q == 19)	//^s to stop and go back to the menu
					{
						break;
					}
					
					U1THR = q;
		  
				}
			  
				if ((U1LSR & 0x01) != 0)	//if something comes in on port1...
				{
					q = U1RBR;	//gotta read the RX buffer to set the flag back to 0
					   
					while((U0LSR & 0x40) != 0x40);	//bit 6 in U0LSR is TEMT
					U0THR = q;
					  
				}
			}
		}
		
		
		
	}
        
        
        
}

void get_adc(void)
{
	int temp;
	
	AD0CR = 0x0020FF02;	//AD0.1, P0.28
	AD0CR |= 0x01000000;	//start conversion
	
	while ((temp & 0x80000000) == 0)
	{
		temp = AD0GDR;
	}
	
	temp &= 0x0000FFC0;
    x_sense = temp / 0x00000040;
	
	AD0CR = 0x00000000;
	
	
	
	AD0CR = 0x0020FF04;	//AD0.2, P0.29
	AD0CR |= 0x01000000;	//start conversion
	
	while ((temp & 0x80000000) == 0)
	{
		temp = AD0GDR;
	}
	
	temp &= 0x0000FFC0;
    y_sense = temp / 0x00000040;
	
	AD0CR = 0x00000000;
	
	
	
	AD0CR = 0x0020FF08;	//AD0.3, P0.30
	AD0CR |= 0x01000000;	//start conversion
	
	while ((temp & 0x80000000) == 0)
	{
		temp = AD0GDR;
	}
	
	temp &= 0x0000FFC0;
    z_sense = temp / 0x00000040;
	
	AD0CR = 0x00000000;
	

}

void set_mux(void)
{
	if (channel == 0)	//GPS
	{
		IOCLR1 = 0x00400000;	//Mux IN1 set low
		IOCLR0 = 0x00010000;	//Mux IN2 set low
	}
	
	else if (channel == 1)	//cell
	{
		IOSET1 = 0x00400000;	//Mux IN1 set high
		IOCLR0 = 0x00010000;	//Mux IN2 set low
	}
	
	else if (channel == 2)	//BT
	{
		IOCLR1 = 0x00400000;	//Mux IN1 set low
		IOSET0 = 0x00010000;	//Mux IN2 set high
	}
	
	else if (channel == 3)	//N/C
	{
		IOSET1 = 0x00400000;	//Mux IN1 set high
		IOSET0 = 0x00010000;	//Mux IN2 set high
	}
	
}

void set_sense(void)
{
	if (sense == 0)	//
	{
		IOCLR1 = 0x00030000;	//GS1 and GS2 set low
	}
	
	else if (sense == 1)	//
	{
		IOSET1 = 0x00020000;	//GS1 set high
		IOCLR1 = 0x00010000;	//GS2 set low
	}
	
	else if (sense == 2)	//
	{
		IOSET1 = 0x00010000;	//GS2 set high
		IOCLR1 = 0x00020000;	//GS1 set low
	}
	
	else if (sense == 3)	//
	{
		IOSET1 = 0x00030000;	//GS1 and GS2 set high
	}
	
}

